## usage: [E,V,Emax,Vmax,Eimg] = convedgemat(Img, kernelsize, gridsize, freq)
##
##	Img: p x q matrix (grayscale image)
##
##		Img: 2D matrix with grayscale image
##		kernelsize: width (==height) of convolution kernel
##		gridsize:   sampling grid size
##		freq: 	    gabor frequency (relative to the kernelsize) 
##
## Returns a n x m matrix E where 
##	n = p / gridsize
##	m = q / gridsize
##	and each E_ij is the edge orientation of the subimage patch.
##
## First, convolve with DoG filter, and then convolve with gabor
## filters of orientations 0, 10, 20, ... , 180. Cut the generated images
## into patches and find the mximum value and the index	
##
##	Usage example:
##		[E,V,Emax,Vmax,Eimg] = convedgemat2(imagemat,9,3,1);
##
##		both DoG and gabor filters are  9 x 9, and
##		Gabor frequency is 1.
##		final grid sampling size is 3 x 3.
##
## $Id: convedgemat.m,v 1.2 2002/03/09 17:31:27 yschoe Exp yschoe $

function [E,V,Emax,Vmax,Eimg] = convedgemat2(Img, kernelsize, gridsize, freq)

	# get image size
	[oldp, oldq] = size(Img);

	# adjust grid size to have the conv kernels
	# centered at the middle: i.e. make the gridsize an odd number.
	if ( rem(kernelsize,2) == 0 ) 
		kernelsize ++;
	endif

	DoG = dog(kernelsize);

        # convolve with DoG, and only use the abs part
	Eimg = real(conv2(Img,DoG,"same"));	

	    
        # calculate new image size to make it divisible by gridsize
	p = floor(oldp/gridsize)*gridsize;
	q = floor(oldq/gridsize)*gridsize;

   	ygrids = floor(p/gridsize);
	xgrids = floor(q/gridsize);

	#initialize matrices
	V = zeros(ygrids,xgrids).-999999;
	E = zeros(ygrids,xgrids);
	
	Vmax=zeros(oldp,oldq);
        Emax=zeros(oldp,oldq);
       
	g = gabor(kernelsize,kernelsize,"sin",0,freq);
	g2 = gabor(kernelsize,kernelsize,"cos",0,freq);

        Vmax=real(conv2(Eimg,g,"same")).^2+real(conv2(Eimg,g2,"same")).^2;
	Emax=Emax+1;
        
        #find Vmax and Emax iteratively
	for k=2:19
             g = gabor(kernelsize,kernelsize,"sin",(k-1)*10,freq);
             g2 = gabor(kernelsize,kernelsize,"cos",(k-1)*10,freq);
	     vtmp= real(conv2(Eimg,g,"same")).^2+real(conv2(Eimg,g2,"same")).^2;
             Vmax=max(Vmax,vtmp);
             Emax=max(Emax,k*(sign(vtmp-Vmax)+1));
        endfor

        #cut Vmax and Emax into grids and calculate V and E
 	for j=1:ygrids
          for i=1:xgrids
		vtmp = vec(Vmax(1+((j-1)*gridsize):(j*gridsize),
			        1+((i-1)*gridsize):(i*gridsize)));
		# idx contains the index of the max within that subimage
		# block.
  	        [V(j,i),idx]=max(vtmp);
		etmp = vec(Emax(1+((j-1)*gridsize):(j*gridsize),
				1+((i-1)*gridsize):(i*gridsize)));
                E(j,i)=etmp(idx);
          endfor
        endfor

endfunction
